------------------------------------------------------------------------------
-- Broker_TSMProfile - Displays the current TSM profile and lets you switch between profiles
------------------------------------------------------------------------------
-- LDB.lua
--
-- Author: Expelliarm5s / July 2020 / All Rights Reserved
--
-- Version 1.2.5
------------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI

local addonName, addon = ...
local LDB = addon:NewModule("LDB", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibLDB = LibStub("LibDataBroker-1.1")
local LibQTip = LibStub("LibQTip-1.0")
local Icon = LibStub("LibDBIcon-1.0")
local private = {}
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Debug Stuff

function LDB:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("LDB~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Addon Loading / Player Login/Logout

function LDB:Login()
	LDB:DebugPrintf("Login()")

	private.LDB = LibLDB:NewDataObject(addonName, {
		type = "data source", -- data source
		label = addonName,
		text = LDB:GetLDBText(),
		icon = "Interface\\Icons\\Racial_Dwarf_FindTreasure",
		OnClick = function(_, button)
			if button == "LeftButton" then
				LDB:DebugPrintf("WARN~Click on LDB")
				local editbox = ChatEdit_ChooseBoxForSend(DEFAULT_CHAT_FRAME)
				ChatEdit_ActivateChat(editbox)
				editbox:SetText("/tsm")
				ChatEdit_OnEnterPressed(editbox)
			end
			if button == "RightButton" then
				LDB:DebugPrintf("WARN~Right click on LDB")
				InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
			end
		end,
		OnEnter = function(this)
			local tooltip = LibQTip:Acquire(addon)
			tooltip:SmartAnchorTo(this)
			tooltip:SetAutoHideDelay(0.1, this)
			tooltip:EnableMouse(true)
			LDB:Update()
			tooltip:Show()
		end
	})

	Icon:Register(addonName, private.LDB, addon.db.global.minimap)
end

function LDB:GetLDBText(status, showAddonName)
	local showName = false or showAddonName

	if showName then
		return L["TSM Profile"] .. (status and (": " .. tostring(status)) or "")
	else
		return status and (tostring(status)) or ""
	end
end

local function tooltip_line_OnMouseUp(frame, profile, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
	end
	if button == "LeftButton" then
		LDB:DebugPrintf("WARN~Click on profile...")
		if TSM_API and TSM_API.GetProfiles and TSM_API.GetActiveProfile and TSM_API.SetActiveProfile then
			LDB:DebugPrintf("  from %s", tostring(TSM_API.GetActiveProfile()))
			for _, aprofile in ipairs(TSM_API.GetProfiles({})) do
				if profile and profile == aprofile and profile ~= TSM_API.GetActiveProfile() then
					TSM_API.SetActiveProfile(profile)
					LDB:DebugPrintf("  to %s", tostring(profile))
					LDB:SecTimer()
					addon:Printf(L["Profile changed to '%s'."], TSM_API.GetActiveProfile())
					LDB:DebugPrintf("OK~" .. L["Profile changed to '%s'."], TSM_API.GetActiveProfile())
				end
			end
		else
			-- fallback
			LDB:DebugPrintf("  switching using fallback method")
			local editbox = ChatEdit_ChooseBoxForSend(DEFAULT_CHAT_FRAME)
			ChatEdit_ActivateChat(editbox)
			editbox:SetText("/tsm profile " .. profile)
			ChatEdit_OnEnterPressed(editbox)
		end
		LDB:Update()
	end
end

function LDB:Update()
	if not private.LDB then return end

	if not LibQTip:IsAcquired(addon) then
		return
	end

	LDB:DebugPrintf("Update")

	-- Tooltip
	local tooltip = LibQTip:Acquire(addon)
	tooltip:Clear()
	tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

	local line = tooltip:AddHeader()
	tooltip:SetCell(line, 1, "|cfffed100" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r", nil, "CENTER", 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "")
	tooltip:SetCell(line, 2, "")
	tooltip:AddSeparator()

	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cfffed100" .. L["Profile"] .. ":|r")
	if TSM_API then
		tooltip:SetCell(line, 2, "|cfffed100" .. L["State"] .. ":|r")
	else
		tooltip:SetCell(line, 2, "")
	end

	if TSM_API and TSM_API.GetProfiles and TSM_API.GetActiveProfile then
		for i, profile in ipairs(TSM_API.GetProfiles({})) do
			line = tooltip:AddLine()
			tooltip:SetCell(line, 1, format("%d. %s", i, profile))
			if (TSM_API.GetActiveProfile() == profile) then
				tooltip:SetCell(line, 2, L["On"])
			else
				tooltip:SetCell(line, 2, "--")
			end
			tooltip:SetLineScript(line, "OnMouseUp", tooltip_line_OnMouseUp, profile)
		end
	end

	tooltip:AddSeparator()
	tooltip:AddLine(format(L["%sClick%s on profile to switch"], ITEM_QUALITY_COLORS[5].hex, FONT_COLOR_CODE_CLOSE))
	tooltip:AddLine(format(L["%sRight Click%s opens options"], ITEM_QUALITY_COLORS[5].hex, FONT_COLOR_CODE_CLOSE))
	tooltip:UpdateScrolling()
end

------------------------------------------------------------------------------
-- Timers

function LDB:SecTimer()
	local profile = "???"
	if TSM_API and TSM_API.GetActiveProfile then
		profile = TSM_API.GetActiveProfile()
	end
	private.LDB.text = LDB:GetLDBText(profile)

	if private.lastProfile and private.lastProfile ~= profile then
		addon.Options.PlaySound(addon.db.global.nwSound)
		private.lastProfile = profile
	else
		private.lastProfile = profile
	end
end

-- EOF
